# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import jenkinsapi
import logging
import re
import yaml

from django.conf import settings

from ci.util.cu2d_stack import create_cu2d_stack_from_config
from ci.util.ci_project import create_ci_project
from ci.util.jenkins_job import update_jenkins_job
from ci.models import CiProject

from common.management import JenkinsBaseCommand, jenkins_get

logger = logging.getLogger("qa_dashboard")

# Job name which contains the data dump
CUPSTREAM2DISTRO_CONFIG_DATA_SOURCE = 'dump-cupstream2distro-config'

# List of stack files that shouldn't be consumed
CONFIG_BLACKLIST = ['ubuntu-touch-coreapps']


def config_is_on_blacklist(config_name):
    """Return a bool if we find this filepath in the blacklist."""

    return config_name in CONFIG_BLACKLIST


class Command(JenkinsBaseCommand):
    help = "Slurp all cu2d configs in a repo and deposit to database."

    # for the moment just get everything
    job_regex = re.compile(ur'^.*$')

    def _create_stack(self, stackcfg, release):
        """Creates a temporary dict for Cu2dStack creation"""

        name = stackcfg['name']
        stack_name = '{}.{}'.format(release, name)
        stack_config = dict()
        stack_config['name'] = stack_name
        stack_config['release'] = release
        stack_config['series'] = stackcfg.get('series', '')
        stack_config['ppa'] = stackcfg.get('ppa', '')
        stack_config['server'] = settings.JENKINS_URL
        stack_config['job'] = '{}-{}-{}'.format('cu2d', name, release)
        logger.debug("Slurping stack: {}".format(stack_name))
        logger.debug("Using job: {}".format(stack_config['job']))
        create_cu2d_stack_from_config(stack_config)
        return stack_name

    def _create_project(self,
                        stackcfg,
                        release,
                        stack_name,
                        project_name):
        """Creates a temporary dict for CiProject creation"""

        project_config = dict()
        # TODO: Replace with server from jenkins lookup table when available
        project_config['server'] = settings.JENKINS_URL
        project_config['release'] = release
        project_config['stack'] = stack_name
        project_config['name'] = "{}.{}".format(stack_name, project_name)

        project_data = stackcfg['projects'][project_name]
        project_config['ci'] = project_data.get('ci', None)
        project_config['autolanding'] = project_data.get('autolanding', None)
        logger.debug("Slurping project: {}".format(project_name))
        logger.debug("Ci job: {}".format(project_config['ci']))
        logger.debug(
            "Autolanding job: {}".format(project_config['autolanding']),
        )
        create_ci_project(project_config)

    def _extract_release_and_config_name(self, url):
        """Returns a string pair representing the release and config name.

        http://jenkins-url/blah/blah/{release}.{config}.yaml
        """

        parts = url.split('/')
        (release, config, extension) = parts[-1].split('.')
        return (release, config)

    def update_projects(self, project_list=None):
        if project_list is None:
            project_list = CiProject.objects.all()
        for project in project_list:
            update_jenkins_job(project.jenkins_job)

    def _load_artifact(self, url):
        page = jenkins_get(url, as_json=False)
        return yaml.safe_load(page)

    def _get_artifact_url_list(self):
        # Get the jenkins job which contains the data dump
        jenkapi = jenkinsapi.api.Jenkins(settings.JENKINS_URL)
        job = jenkapi.get_job(CUPSTREAM2DISTRO_CONFIG_DATA_SOURCE)
        build_ids = job.get_build_ids()

        # Get a list of build artificts, each corresponds to a stack file
        artifact_list = []
        for build_id in build_ids:
            build = job.get_build(build_id)
            if build.get_status() == 'SUCCESS':
                for artifact in build.get_artifacts():
                    artifact_list.append(artifact.url)
                # We only need to look at the most recent successful build
                break
        return artifact_list

    def initial_run(self):
        url_list = self._get_artifact_url_list()

        # For each artifact
        for url in url_list:
            # Parse the stack release and name from the url
            (release, config_name) = self._extract_release_and_config_name(url)
            if config_is_on_blacklist(config_name):
                logger.debug("Skipping cu2d config at {} due to blacklist "
                             "match.".format(config_name))
                continue

            data = self._load_artifact(url)
            # Process the stack
            if not data:
                # Stack has no contents
                logger.debug("No projects found in {}".format(url))
                continue
            if 'projects' not in data:
                # Stack has no projects
                logger.debug("No projects found in {}".format(url))
                continue
            stack_name = self._create_stack(data, release)

            # Process the projects
            if stack_name:
                for project_name in data['projects']:
                    self._create_project(data, release, stack_name,
                                         project_name)
        return

    def handle(self, *args, **options):
        if options.get('initial_run'):
            self.initial_run()
        else:
            self.update_projects(project_list=args)
        # TODO ultimately we'll integrate with qa-dashboard services
        #super(Command, self).handle(*args, **options)
