# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import yaml
from django.db import models, IntegrityError, transaction
import util.extractor as extractor
import util.heuristics as heuristics
from launchpadlib.launchpad import Launchpad
from lxml.etree import XMLSyntaxError
#import common.models

logger = logging.getLogger("qa_dashboard")


BLACKLIST_JOBS = [
    'generic-dput',
    'generic-land',
    'generic-update_mp',
    'phablet-dput',
    'phablet-land',
    'generic-mediumtests',
    'generic-cleanup',
    'generic-release-land',
    '1.1prepare',
]


class JenkinsJob(models.Model):
    """A Jenkins job."""

    #enabled = models.BooleanField()
    name = models.CharField(max_length=256, unique=True)
    url = models.URLField()
    project = models.ForeignKey('CiProject', related_name='jenkins_jobs', null=True)
    upstream_job = models.ForeignKey('JenkinsJob', related_name='downstream_jobs', null=True)
    buildable = models.BooleanField()
    in_queue = models.BooleanField()
    # TODO it would be nice if these were all ForeignKeys, could rename as last_build_number, e.g.
    last_build = models.IntegerField(null=True)
    last_completed_build = models.IntegerField(null=True)
    last_failed_build = models.IntegerField(null=True)
    last_successful_build = models.IntegerField(null=True)

    @property
    def timestamp(self):
        try:
            return self.get_last_successful_build().timestamp
        except AttributeError:
            return None

    @property
    def line_coverage(self):
        try:
            return self.get_last_successful_build().coverage_report.line_rate
        except AttributeError:
            return None

    @property
    def branch_coverage(self):
        try:
            return self.get_last_successful_build().coverage_report.branch_rate
        except AttributeError:
            return None

    @property
    def line_count(self):
        try:
            return self.get_last_successful_build().coverage_report.line_count
        except AttributeError:
            logger.debug("line_count AttributeError in JenkinsJob")
            return None

    @property
    def hit_count(self):
        try:
            return self.get_last_successful_build().coverage_report.hit_count
        except AttributeError:
            logger.debug('hit_count AttributeError in JenkinsJob')
            return None

    @property
    def taken_count(self):
        try:
            return self.get_last_successful_build().coverage_report.taken_count
        except AttributeError:
            logger.debug('taken_count AttributeError in JenkinsJob')
            return None

    @property
    def total_count(self):
        try:
            return self.get_last_successful_build().coverage_report.total_count
        except AttributeError:
            logger.debug('total_count AttributeError in JenkinsJob')
            return None

    def get_last_build(self):
        """Get the last successful build as recorded by Jenkins."""
        # TODO whatif we don't have this build in the DB?
        # TODO what about search for the last with status='SUCCESS'?
        # TODO see above about making this an object-property at slurp time
        try:
            return JenkinsBuild.objects.get(
                jenkins_job=self,
                number=self.last_build)
        except JenkinsBuild.DoesNotExist:
            return None

    def get_last_successful_build(self):
        """Get the last successful build as recorded by Jenkins."""
        # TODO whatif we don't have this build in the DB?
        # TODO what about search for the last with status='SUCCESS'?
        # TODO see above about making this an object-property at slurp time
        try:
            return JenkinsBuild.objects.get(
                jenkins_job=self,
                number=self.last_successful_build)
        except JenkinsBuild.DoesNotExist:
            return None

    def _get_last_fail_count(self):
        try:
            return self.get_last_successful_build().test_report.fail_count
        except AttributeError:
            return None
    last_fail_count = property(_get_last_fail_count)

    def _get_last_pass_count(self):
        try:
            return self.get_last_successful_build().test_report.pass_count
        except AttributeError:
            return None
    last_pass_count = property(_get_last_pass_count)

    def _get_last_skip_count(self):
        try:
            return self.get_last_successful_build().test_report.skip_count
        except AttributeError:
            return None
    last_skip_count = property(_get_last_skip_count)

    def _get_last_duration(self):
        try:
            return self.get_last_successful_build().test_report.duration
        except AttributeError:
            return None
    last_duration = property(_get_last_duration)

    def __unicode__(self):
        return self.name


class CiProject(models.Model):
    """Our first-class object which links Jenkins to Launchpad."""

    name = models.CharField(max_length=256, unique=True)
    release = models.ForeignKey('Release', null=True)
    stack = models.ForeignKey('Cu2dStack', null=True)
    ci_primary_job = models.ForeignKey(
        'JenkinsJob', related_name='ci_primary', null=True)
    autolanding_primary_job = models.ForeignKey(
        'JenkinsJob', related_name='autolanding_primary', null=True)
    launchpad_project = models.ForeignKey(
        'LaunchpadProject',
        null=True)

    @property
    def line_coverage(self):
        if self.last_successful_build is not None:
            return self.last_successful_build.line_coverage
        return None

    @property
    def branch_coverage(self):
        if self.last_successful_build is not None:
            return self.last_successful_build.branch_coverage
        return None

    @property
    def line_count(self):
        if self.last_successful_build is not None:
            return self.last_successful_build.line_count

    @property
    def hit_count(self):
        if self.last_successful_build is not None:
            return self.last_successful_build.hit_count

    @property
    def taken_count(self):
        if self.last_successful_build is not None:
            return self.last_successful_build.taken_count

    @property
    def total_count(self):
        if self.last_successful_build is not None:
            return self.last_successful_build.total_count

    @property
    def last_build(self):
        return JenkinsBuild.objects.filter(
            jenkins_job=self.autolanding_primary_job).order_by('timestamp')[0]

    @property
    def last_successful_build(self):
        try:
            return JenkinsBuild.objects.filter(
                jenkins_job=self.autolanding_primary_job,
                number=self.autolanding_primary_job.last_successful_build)[0]
        except IndexError:
            # no builds no biggie
            return None

    @property
    def timestamp(self):
        return self.last_successful_build.timestamp

    def _get_last_fail_count(self):
        try:
            return self.last_successful_build.test_report.fail_count
        except AttributeError:
            return None
        except IndexError:
            return None
    last_fail_count = property(_get_last_fail_count)

    def _get_last_pass_count(self):
        try:
            return self.last_successful_build.test_report.pass_count
        except AttributeError:
            return None
        except IndexError:
            return None
    last_pass_count = property(_get_last_pass_count)

    def _get_last_skip_count(self):
        try:
            return self.last_successful_build.test_report.skip_count
        except AttributeError:
            return None
        except IndexError:
            return None
    last_skip_count = property(_get_last_skip_count)

    def _get_last_duration(self):
        try:
            return self.last_successful_build.test_report.duration
        except AttributeError:
            return None
        except IndexError:
            return None
    last_duration = property(_get_last_duration)

    @property
    def short_name(self):
        """The project name within its stack, e.g. 'lightdm' for "head.mir.lightdm"."""

        # TODO really this should be refactored with each term as a foreign key, e.g.
        return self.name.split('.')[-1]

    def last_coverage_build(self, timestamp):
        """Returns the last build before timestamp reporting test coverage."""

        try:
            return JenkinsBuild.objects.filter(
                jenkins_job=self.autolanding_primary_job,
                coverage_report__isnull=False,
                timestamp__lte=timestamp)[0]
        except IndexError:
            return None

    def __unicode__(self):
        return self.name


class LaunchpadProject(models.Model):
    """Corresponds to a top-level Launchpad project."""

    name = models.CharField(max_length=256)

    @property
    def ci_project(self):
        try:
            head_ci_project = CiProject.objects.get(
                launchpad_project=self,
                release__name='head')
            return head_ci_project
        # TODO: more complicated logic for dormant projects?
        except CiProject.DoesNotExist:
            return None

    def __unicode__(self):
        return self.name


class Cu2dStack(models.Model):
    """Canonical upstream to distro stack.

    An agglomeration of projects often released together.
    """

    name = models.CharField(max_length=64)
    series = models.ForeignKey('Series', null=True)
    ppa = models.CharField(max_length=64, null=True)
    release = models.ForeignKey('Release', null=True)
    integration_job = models.ForeignKey(
        'JenkinsJob', related_name='integration', null=True)

    @property
    def status(self):
        return heuristics.get_integration_status(
            JenkinsJob, JenkinsBuild, self.last_build)

    @property
    def last_publish(self):
        return heuristics.get_integration_last_publish(
            JenkinsJob, JenkinsBuild, self.integration_job)

    @property
    def last_pass_count(self):
        pass_count = heuristics.get_integration_last_pass_count(
            JenkinsJob, JenkinsBuild, self.integration_job)
        return pass_count

    @property
    def last_fail_count(self):
        fail_count = heuristics.get_integration_last_fail_count(
            JenkinsJob, JenkinsBuild, self.integration_job)
        return fail_count

    @property
    def last_total_count(self):
        total_count = heuristics.get_integration_last_total_count(
            JenkinsJob, JenkinsBuild, self.integration_job)
        return total_count

    @property
    def last_pass_rate(self):
        pass_count = self.last_pass_count
        total_count = self.last_total_count
        return float(pass_count) / float(total_count) if pass_count > 0 else 0

    @property
    def last_build(self):
        return JenkinsBuild.objects.get(
            jenkins_job=self.integration_job,
            number=self.integration_job.last_build)

    @property
    def last_successful_build(self):
        return JenkinsBuild.objects.get(
            jenkins_job=self.integration_job,
            number=self.integration_job.last_successful_build)

    @property
    def build_list(self):
        return JenkinsBuild.objects.filter(
            jenkins_job=self.integration_job)

    def __unicode__(self):
        return self.name


class JenkinsTestReport(models.Model):
    fail_count = models.IntegerField()
    pass_count = models.IntegerField()
    skip_count = models.IntegerField()
    duration = models.IntegerField()  # milliseconds


class JenkinsCoverageReport(models.Model):
    jenkins_build = models.ForeignKey('JenkinsBuild', null=True, unique=True)
    line_count = models.IntegerField()
    hit_count = models.IntegerField()
    branch_count = models.IntegerField()
    taken_count = models.IntegerField()
    total_count = models.IntegerField()

    @property
    def line_rate(self):
        try:
            return float(self.hit_count) / float(self.line_count)
        except ZeroDivisionError:
            return 0

    @property
    def branch_rate(self):
        try:
            return float(self.taken_count) / float(self.total_count)
        except ZeroDivisionError:
            return 0


class JenkinsFlavor(models.Model):
    name = models.CharField(max_length=64)

    def __unicode__(self):
        return self.name


class Release(models.Model):
    name = models.CharField(max_length=64)

    def __unicode__(self):
        return self.name


class Series(models.Model):
    name = models.CharField(max_length=64)

    def __unicode__(self):
        return self.name


class JenkinsResult(models.Model):
    name = models.CharField(max_length=64)

    def __unicode__(self):
        return self.name


class JenkinsRun(models.Model):

    jenkins_build = models.ForeignKey('JenkinsBuild',
                                      related_name='runs',
                                      null=True)
    number = models.IntegerField(null=True)
    name = models.CharField(max_length=256)
    url = models.URLField(unique=True)
    duration = models.IntegerField() # seconds
    flavor = models.ForeignKey('JenkinsFlavor')
    series = models.ForeignKey('Series')
    result = models.ForeignKey('JenkinsResult')
    coverage_report = models.ForeignKey('JenkinsCoverageReport', null=True)
    test_report = models.ForeignKey('JenkinsTestReport', null=True)
    timestamp = models.DateTimeField()

    def __unicode__(self):
        return self.name


class JenkinsBuild(models.Model):
    """A Jenkins build."""

    # TODO I think these are properties (?)
    #coverage_report
    #flavors
    #parameters
    #series
    #test_reports
    jenkins_job = models.ForeignKey('JenkinsJob', related_name='builds', null=True)
    number = models.IntegerField()
    timestamp = models.DateTimeField()
    url = models.URLField(unique=True)
    result = models.ForeignKey('JenkinsResult')
    downstream_builds = models.ManyToManyField('JenkinsBuild')
    coverage_report = models.ForeignKey(JenkinsCoverageReport, null=True)
    test_report = models.ForeignKey(JenkinsTestReport, null=True)

    @property
    def line_coverage(self):
        if self.coverage_report:
            return self.coverage_report.line_rate
        return None

    @property
    def branch_coverage(self):
        if self.coverage_report:
            return self.coverage_report.branch_rate
        return None

    @property
    def line_count(self):
        if self.coverage_report:
            return self.coverage_report.line_count
        return None

    @property
    def hit_count(self):
        if self.coverage_report:
            return self.coverage_report.hit_count

    @property
    def taken_count(self):
        if self.coverage_report:
            return self.coverage_report.taken_count

    @property
    def total_count(self):
        if self.coverage_report:
            return self.coverage_report.total_count

    def __unicode__(self):
        return self.url
