# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
from mock import patch, MagicMock
from urlparse import urljoin
from nose.plugins.attrib import attr

from django.conf import settings
from django.test import TestCase
from ci.util.extractor import (BuildUrl,
                               get_console_text_from_url,
                               get_downstream_builds_from_build_url,
                               get_downstream_builds_from_console_text)


CONSOLE_TEXT = """Started by user ps-jenkins
Building on master
Waiting for the completion of mir-clang-raring-amd64-build
mir-clang-raring-amd64-build #84 completed. Result was SUCCESS
Waiting for the completion of mir-android-raring-i386-build
mir-android-raring-i386-build #201 completed. Result was SUCCESS
Waiting for the completion of mir-quantal-amd64-autolanding
mir-quantal-amd64-autolanding #100 completed. Result was SUCCESS
Build step 'Trigger/call builds on other projects' changed build result to SUCCESS
Waiting for the completion of generic-land
generic-land #6943 completed. Result was SUCCESS
Build was marked for publishing on https://jenkins.qa.ubuntu.com/
Triggering a new build of mir-vm-autolanding-build #306
Finished: SUCCESS
"""

DOWNSTREAM_BUILDS = [
    BuildUrl(
        number=84,
        url=urljoin(
            settings.JENKINS_URL,
            'job/mir-clang-raring-amd64-build/84',
        ),
    ),
    BuildUrl(
        number=201,
        url=urljoin(
            settings.JENKINS_URL,
            'job/mir-android-raring-i386-build/201',
        ),
    ),
    BuildUrl(
        number=100,
        url=urljoin(
            settings.JENKINS_URL,
            'job/mir-quantal-amd64-autolanding/100'),
    ),
    BuildUrl(
        number=6943,
        url=urljoin(settings.JENKINS_URL, 'job/generic-land/6943')),
    BuildUrl(
        number=306,
        url=urljoin(settings.JENKINS_URL, 'job/mir-vm-autolanding-build/306')),
]


@attr('qa_lab')
class TestGetConsoleTextForBuild(TestCase):
    """Integration test."""

    def test_vanilla(self):
        build_url = urljoin(settings.JENKINS_URL, "job/mir-autolanding/99/")
        result = get_console_text_from_url(build_url)
        self.assertEqual(CONSOLE_TEXT, result)


class TestGetDownstreamBuildsFromBuildUrl(TestCase):

    def test_vanilla(self):
        get_console_text_from_url_patch = MagicMock(return_value=CONSOLE_TEXT)
        with patch('ci.util.extractor.get_console_text_from_url',
                   get_console_text_from_url_patch):
            result = get_downstream_builds_from_build_url(
                urljoin(settings.JENKINS_URL, 'job/mir-autolanding/'))
        self.assertEqual(DOWNSTREAM_BUILDS, result)


class TestMatchDownstreamBuildLine(TestCase):

    line = "mir-clang-raring-amd64-build #84 completed. Result was SUCCESS"
    build_regex = re.compile("^(?P<name>.*) #(?P<number>\d+) completed\. Result was .*$")

    def test_we_find_something(self):
        self.assertIsNotNone(self.build_regex.match(self.line))

    def test_match_is_equal_to_build_name(self):
        result = self.build_regex.match(self.line)
        if result is not None:
            self.assertEqual('mir-clang-raring-amd64-build',
                             result.group('name'))
        else:
            self.fail("no regular expression match")

    def test_match_is_equal_to_build_number(self):
        result = self.build_regex.match(self.line)
        if result is not None:
            self.assertEqual('84',
                             result.group('number'))
        else:
            self.fail("no regular expression match")


class TestMatchDownstreamBuildText(TestCase):

    build_regex = re.compile("^(?P<name>.*) #(?P<number>\d+) completed\. Result was .*$",
                             re.MULTILINE)

    def test_we_find_something(self):
        result = self.build_regex.search(CONSOLE_TEXT)
        self.assertIsNotNone(result)

    def test_correct_number_of_result_groups(self):
        result = self.build_regex.findall(CONSOLE_TEXT)
        if result is not None:
            self.assertEqual(4, len(result))
        else:
            self.fail("no result groups found")
