# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
import datetime
import json
import os.path
import unittest

from ci.util.extractor import JenkinsRun
import utilities

"""Tests for class JenkinsRun"""

DATA_PATH = "ci/tests/data"
DATA_DICT = {'amd64': {'url': 'http://10.97.0.6:8080/job/nux-mbs-'
                       'autolanding/build=pbuilder,distribution=raring,'
                       'flavor=amd64/98',
                       'file': 'nux-mbs-autolanding-98-amd64.json',
                      },
             'armhf': {'url': 'http://10.97.0.6:8080/job/nux-mbs-'
                       'autolanding/build=panda-pbuilder,'
                       'distribution=raring,flavor=armhf/98',
                       'file': 'nux-mbs-autolanding-98-armhf.json',
                      },
             'i386': {'url': 'http://10.97.0.6:8080/job/nux-mbs-autolanding/'
                      'build=pbuilder,distribution=raring,flavor=i386/98',
                      'file': 'nux-mbs-autolanding-98-i386.json',
                     },
             'compiz': {'url': 'http://10.97.0.6:8080/job/compiz-mbs-'
                        'autolanding/build=panda-pbuilder,'
                        'distribution=raring,flavor=armhf/193',
                        'file': 'compiz-mbs-autolanding-193-armhf.json',
                      },
             'testReport': {'url': 'http://10.97.0.6:8080/job/nux-mbs-'
                            'autolanding/build=pbuilder,distribution=raring,'
                            'flavor=amd64/98/testReport',
                            'file': 'nux-mbs-autolanding-98-amd64-'
                            'testReport.json',
                      },
             'coverageReport': {'url': 'http://10.97.0.6:8080/job/nux-mbs-'
                                'autolanding/build=pbuilder,'
                                'distribution=raring,flavor=amd64/98/'
                                'artifacts/work/results/coverage.xml',
                                'file': 'nux-mbs-autolanding-98-i386.'
                                'coverage.xml',
                      },
            }


class JenkinsRunTestCase(unittest.TestCase):
    """Unit tests for class JenkinsRun"""

    def _get_data(self, key):
        return {'json': utilities.get_json_data(DATA_PATH,
                                                DATA_DICT[key]['file']),
                'url': DATA_DICT[key]['url']}

    def _get_text(self, key):
        return {'text': utilities.get_text(DATA_PATH,
                                           DATA_DICT[key]['file']),
                'url': DATA_DICT[key]['url']}

    def test_str(self):
        data = self._get_data('compiz')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertIn("<JenkinsRun: ", "%s" % jr)
        self.assertIn("compiz-mbs-autolanding", "%s" % jr)

    def test_name(self):
        data = self._get_data('amd64')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.name,
                         u"nux-mbs-autolanding \xbb pbuilder,raring,amd64 "
                         "#98")

    def test_number(self):
        data = self._get_data('i386')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.number, 98)

    def test_duration(self):
        data = self._get_data('i386')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.duration.seconds,
                         datetime.timedelta(seconds=39, minutes=23).seconds)

    def test_timestamp(self):
        data = self._get_data('armhf')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.timestamp,
                         datetime.datetime(2013, 1, 11, 10, 52, 52))

    def test_flavor(self):
        flavor = 'amd64'
        data = self._get_data(flavor)
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.flavor, flavor)

    def test_series(self):
        data = self._get_data('armhf')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.series, 'raring')

    def test_result(self):
        data = self._get_data('amd64')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.result, "SUCCESS")

    def test_result_null(self):
        data = self._get_data("compiz")
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.result, None)

    def test_building(self):
        data = self._get_data('i386')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.building, False)

    def test_building_true(self):
        data = self._get_data("compiz")
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.building, True)

    def test_parameters(self):
        data = self._get_data('amd64')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(len(jr.parameters), 19)

    def test_test_report(self):
        data = self._get_data('amd64')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.get_test_report_url(),
                         'http://10.97.0.6:8080/job/nux-mbs-autolanding/'
                         'build=pbuilder,distribution=raring,flavor=amd64/98/'
                         'testReport')

    def test_coverage_report(self):
        data = self._get_data('i386')
        jr = JenkinsRun(data['url'], data['json'])
        self.assertEqual(jr.get_coverage_report_url(),
                         'http://10.97.0.6:8080/job/nux-mbs-autolanding/'
                         'build=pbuilder,distribution=raring,flavor=i386/98/'
                         'artifact/work/results/coverage.xml')
