# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import re
from mock import patch, MagicMock
from django.test import TestCase
import common
from ci.management.commands.jenkins_pull_ci import (config_is_on_blacklist,
                                                    Command)


class TestConfigIsOnBlackList(TestCase):
    
    def test_yes(self):
        config_blacklist = ['ubuntu-touch-coreapps']
        config_name = "ubuntu-touch-coreapps"
        with patch('ci.management.commands.jenkins_pull_ci.CONFIG_BLACKLIST',
                   config_blacklist):
            result = config_is_on_blacklist(config_name)
        self.assertTrue(result)


    def test_no(self):
        config_blacklist = ['ubuntu-touch-coreapps']
        config_name = "apps"
        with patch('ci.management.commands.jenkins_pull_ci.CONFIG_BLACKLIST',
                   config_blacklist):
            result = config_is_on_blacklist(config_name)
        self.assertFalse(result)


class JenkinsPullCiCommandTestCase(TestCase):

    def setUp(self):
        command_mock = MagicMock(
            spec=Command,
            return_value=None)
        self.command_patch = patch.object(
            Command,
            '__init__',
            new=command_mock)
        self.command_patch.start()
        self.command = Command()
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()


    def tearDown(self):
        self.command_patch.stop()
        self.logger_patch.stop()


class TestJenkinsPullCiCommand(JenkinsPullCiCommandTestCase):

    def test_handle_initial_run(self):
        with patch.object(self.command, 'initial_run') as initial_run:
            result = self.command.handle(initial_run=True)
        self.assertTrue(initial_run.called)

    def test_handle_update_projects_none(self):
        with patch.object(self.command, 'update_projects') as update_projects:
            result = self.command.handle()
        update_projects.assert_called_with(project_list=())

    def test_handle_update_projects_some(self):
        with patch.object(self.command, 'update_projects') as update_projects:
            result = self.command.handle('fake_project')
        update_projects.assert_called_with(project_list=('fake_project',))


class TestInitialRun(JenkinsPullCiCommandTestCase):
    def test_no_urls(self):
        self.command._get_artifact_url_list = MagicMock(return_value=[])
        self.command._load_artifact = MagicMock()
        result = self.command.initial_run()
        self.assertFalse(self.command._load_artifact.called)

    def test_no_data(self):
        self.command._get_artifact_url_list = MagicMock(
            return_value=['http://jenkins/blah/blah/release.stack.yaml'])
        self.command._load_artifact = MagicMock(return_value=None)
        self.command._create_stack = MagicMock()
        self.command._create_project = MagicMock()
        result = self.command.initial_run()
        self.command._load_artifact.assert_called()
        self.assertFalse(self.command._create_stack.called)
        self.assertFalse(self.command._create_project.called)

    def test_no_projects(self):
        stack = {'name': 'fake'}
        self.command._get_artifact_url_list = MagicMock(
            return_value=['http://jenkins/blah/blah/release.stack.yaml'])
        self.command._load_artifact = MagicMock(return_value=stack)
        self.command._create_stack = MagicMock()
        self.command._create_project = MagicMock()
        result = self.command.initial_run()
        self.command._load_artifact.assert_called()
        self.assertFalse(self.command._create_stack.called)
        self.assertFalse(self.command._create_project.called)

    def test_simple_stack(self):
        stack = {'name': 'fake',
                 'ppa': 'ppa:project/ppa',
                 'series': 'saucy',
                 'projects': {
                     'project-1': {
                         'ci': 'project-1-ci',
                         'autolanding': 'project-1-autolanding'}}}
        self.command._get_artifact_url_list = MagicMock(
            return_value=['http://jenkins/blah/blah/release.stack.yaml'])
        self.command._load_artifact = MagicMock(return_value=stack)
        self.command._create_stack = MagicMock()
        self.command._create_project = MagicMock()
        result = self.command.initial_run()
        self.command._load_artifact.assert_called()
        self.command._create_stack.assert_called()
        self.command._create_project.assert_called()


class TestExtractReleaseAndConfigName(JenkinsPullCiCommandTestCase):

    def test_vanilla(self):
        (release, config) = self.command._extract_release_and_config_name(
            "http:///tmp/tmp_dir/stack/raring.stack.cfg")
        self.assertEqual('raring', release)
        self.assertEqual('stack', config)


class TestCreateStack(JenkinsPullCiCommandTestCase):

    def test_vanilla(self):
        fake_stackcfg = {
            'name': 'fake_stack_name',
            'series': 'fake series',
            'ppa': 'fake_ppa',
            'server': 'http://10.97.0.1:8080',
            'job': 'cu2d-fake_stack_name-fake_release'}
        with patch('ci.management.commands.jenkins_pull_ci.create_cu2d_stack_from_config') \
                as create_cu2d_stack_from_config:
            result = self.command._create_stack(
                fake_stackcfg,
                "fake_release")
        fake_stackcfg['name'] = 'fake_release.fake_stack_name'
        fake_stackcfg['release'] = 'fake_release'
        create_cu2d_stack_from_config.assert_called_with(fake_stackcfg)


class TestUpdateProjects(JenkinsPullCiCommandTestCase):

    def test_update_everything(self):
        fake_project_list = [MagicMock(jenkins_job="fake_jenkins_job"),
                             MagicMock(jenkins_job="also_fake_jenkins_job")]
        ci_project_objects_all = MagicMock(return_value=fake_project_list)
        with patch('ci.models.CiProject.objects.all',
                   ci_project_objects_all), \
                patch('ci.management.commands.jenkins_pull_ci.update_jenkins_job') as update_jenkins_job:
            self.command.update_projects()
        update_jenkins_job.assert_called_with('also_fake_jenkins_job')
        self.assertEqual(2, update_jenkins_job.call_count)


    def test_update_one_project(self):
        with patch('ci.models.CiProject.objects.all') as ci_project_objects_all, \
                patch('ci.management.commands.jenkins_pull_ci.update_jenkins_job') as update_jenkins_job:
            self.command.update_projects([MagicMock(jenkins_job="fake_jenkins_job",)])
        update_jenkins_job.assert_called_with('fake_jenkins_job')
        self.assertFalse(ci_project_objects_all.called)

