# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock

from django.test import TestCase
from ci.util import heuristics


class TestHeuristics(TestCase):

    def setUp(self):
        self.JenkinsJob_mock = MagicMock()
        self.JenkinsBuild_mock = MagicMock()
        self.prepare_mock = self._create_build('foo-prepare', 'SUCCESS')
        self.build_mock = self._create_build('foo-build', 'SUCCESS')
        self.check_mock = self._create_build('foo-check', 'SUCCESS')
        self.publish_mock = self._create_build('foo-publish', 'SUCCESS')
        self.build_set = [self.prepare_mock,
                          self.build_mock,
                          self.check_mock,
                          self.publish_mock]
        self.integration_build = MagicMock()
        self.integration_build.downstream_builds = MagicMock()
        self.integration_build.downstream_builds.all = MagicMock(
            return_value=self.build_set)
        self.integration_job = MagicMock()

    def _create_build(self, name, status):
        build_mock = MagicMock()
        build_mock.jenkins_job = MagicMock()
        build_mock.jenkins_job.name = name
        build_mock.result = MagicMock()
        build_mock.result.name = status
        return build_mock

    def test_get_integration_status_published(self):
        ret = heuristics.get_integration_status(self.JenkinsJob_mock,
                                                self.JenkinsBuild_mock,
                                                self.integration_build)
        self.assertEqual(ret, 'Published')

    def test_get_integration_status_fail_prepare(self):
        self.prepare_mock.result.name = "FAILED"
        ret = heuristics.get_integration_status(self.JenkinsJob_mock,
                                                self.JenkinsBuild_mock,
                                                self.integration_build)
        self.assertEqual(ret, 'Failed prepare')

    def test_get_integration_status_fail_build(self):
        self.build_mock.result.name = "FAILED"
        ret = heuristics.get_integration_status(self.JenkinsJob_mock,
                                                self.JenkinsBuild_mock,
                                                self.integration_build)
        self.assertEqual(ret, 'Failed build')

    def test_get_integration_status_fail_check(self):
        self.check_mock.result.name = "FAILED"
        ret = heuristics.get_integration_status(self.JenkinsJob_mock,
                                                self.JenkinsBuild_mock,
                                                self.integration_build)
        self.assertEqual(ret, 'Failed check')

    def test_get_integration_status_fail_publish(self):
        self.publish_mock.result.name = "FAILED"
        ret = heuristics.get_integration_status(self.JenkinsJob_mock,
                                                self.JenkinsBuild_mock,
                                                self.integration_build)
        self.assertEqual(ret, 'Failed publish')

    def test_get_integration_status_manual_publish(self):
        self.publish_mock.result.name = "FAILED"
        manual_publish_job_mock = MagicMock()
        manual_publish_job_mock.name = 'manual-publish'
        manual_publish_build_mock = MagicMock()
        manual_publish_build_mock.timestamp = 2
        JenkinsJob_mock = MagicMock()
        JenkinsJob_mock.objects = MagicMock()
        JenkinsJob_mock.objects.filter = MagicMock(
            return_value=[manual_publish_job_mock])
        JenkinsBuild_mock = MagicMock()
        JenkinsBuild_mock.objects = MagicMock()
        JenkinsBuild_mock.objects.get = MagicMock(
            return_value=manual_publish_build_mock)
        self.integration_build.timestamp = 1
        ret = heuristics.get_integration_status(JenkinsJob_mock,
                                                JenkinsBuild_mock,
                                                self.integration_build)
        self.assertEqual(ret, 'Manual publish')

    def test_get_integration_status_no_manual_publish(self):
        self.publish_mock.result.name = "FAILED"
        manual_publish_job_mock = MagicMock()
        manual_publish_job_mock.name = 'manual-publish'
        manual_publish_build_mock = MagicMock()
        manual_publish_build_mock.timestamp = 1
        JenkinsJob_mock = MagicMock()
        JenkinsJob_mock.objects = MagicMock()
        JenkinsJob_mock.objects.filter = MagicMock(
            return_value=[manual_publish_job_mock])
        JenkinsBuild_mock = MagicMock()
        JenkinsBuild_mock.objects = MagicMock()
        JenkinsBuild_mock.objects.get = MagicMock(
            return_value=manual_publish_build_mock)
        self.integration_build.timestamp = 1
        ret = heuristics.get_integration_status(JenkinsJob_mock,
                                                JenkinsBuild_mock,
                                                self.integration_build)
        self.assertEqual(ret, 'Failed publish')

    def test_get_integration_last_publish(self):
        manual_publish_job_mock = MagicMock()
        manual_publish_job_mock.name = 'manual-publish'
        manual_publish_build_mock = MagicMock()
        manual_publish_build_mock.timestamp = 2
        JenkinsJob_mock = MagicMock()
        JenkinsJob_mock.objects = MagicMock()
        JenkinsJob_mock.objects.filter = MagicMock(
            return_value=[manual_publish_job_mock])
        JenkinsBuild_mock = MagicMock()
        JenkinsBuild_mock.objects = MagicMock()
        JenkinsBuild_mock.objects.get = MagicMock(
            return_value=manual_publish_build_mock)
        ret = heuristics.get_integration_last_publish(JenkinsJob_mock,
                                                      JenkinsBuild_mock,
                                                      self.integration_job)
        self.assertEqual(ret, 2)

    def test_get_integration_last_publish_no_job(self):
        manual_publish_job_mock = MagicMock()
        JenkinsJob_mock = MagicMock()
        JenkinsJob_mock.objects = MagicMock()
        JenkinsJob_mock.objects.filter = MagicMock(return_value=[])
        JenkinsBuild_mock = MagicMock()
        ret = heuristics.get_integration_last_publish(JenkinsJob_mock,
                                                      JenkinsBuild_mock,
                                                      self.integration_job)
        self.assertEqual(ret, None)

    def test_get_integration_last_publish_no_build(self):
        manual_publish_job_mock = MagicMock()
        manual_publish_job_mock.name = 'manual-publish'
        JenkinsJob_mock = MagicMock()
        JenkinsJob_mock.objects = MagicMock()
        JenkinsJob_mock.objects.filter = MagicMock(
            return_value=[manual_publish_job_mock])
        JenkinsBuild_mock = MagicMock()
        JenkinsBuild_mock.objects = MagicMock()
        JenkinsBuild_mock.objects.get = MagicMock(return_value=None)
        ret = heuristics.get_integration_last_publish(JenkinsJob_mock,
                                                      JenkinsBuild_mock,
                                                      self.integration_job)
        self.assertEqual(ret, None)
