# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
from django.test import TestCase
from django.db import IntegrityError
from unittest import skip
from ci.models import JenkinsJob, JenkinsBuild
import ci.util.extractor as extractor
import ci.util.jenkins_build as util_jenkins_build


class JenkinsBuildManagerTestCase(TestCase):

    def setUp(self):
        jenkins_build_objects_create_mock = MagicMock(
            return_value=None)
        self.jenkins_build_objects_create_patch = patch.object(
            JenkinsBuild.objects,
            'create',
            new=jenkins_build_objects_create_mock)
        self.jenkins_build_objects_create = self.jenkins_build_objects_create_patch.start()
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()
        self.dummy_url = "http://this.string.is.actually.a.URL/no/kidding"
        self.dummy_jenkins_job = "a Django representation of a Jenkins job"

    def tearDown(self):
        self.jenkins_build_objects_create_patch.stop()
        self.logger_patch.stop()


class TestCreateFromUrl(JenkinsBuildManagerTestCase):

    def setUp(self):
        super(TestCreateFromUrl, self).setUp()
        self.create_downstream_runs_patch = patch(
            'ci.util.jenkins_build.create_downstream_runs')
        self.create_downstream_runs_patch.start()
        self.create_downstream_builds_patch = patch(
            'ci.util.jenkins_build.create_downstream_builds')
        self.create_downstream_builds_patch.start()
        self.create_cobertura_report_patch = patch(
            'ci.util.jenkins_build.create_cobertura_report')
        self.create_cobertura_report_patch.start()
        self.create_jenkins_test_report_patch = patch(
            'ci.util.jenkins_build.create_jenkins_test_report')
        self.create_jenkins_test_report_patch.start()
        self.aggregate_coverage_report_patch = patch(
            'ci.util.jenkins_build.aggregate_coverage_report')
        self.aggregate_coverage_report_patch.start()
        self.aggregate_test_report_patch = patch(
            'ci.util.jenkins_build.aggregate_test_report')
        self.aggregate_test_report_patch.start()

    def tearDown(self):
        self.create_downstream_runs_patch.stop()
        self.create_downstream_builds_patch.stop()
        self.create_cobertura_report_patch.stop()
        self.create_jenkins_test_report_patch.stop()
        self.aggregate_coverage_report_patch.stop()
        self.aggregate_test_report_patch.stop()
        super(TestCreateFromUrl, self).tearDown()

    def test_vanilla(self):
        get_text_from_url = MagicMock(side_effect=EnvironmentError)
        get_json_from_url = MagicMock(side_effect=EnvironmentError)
        jenkins_result_get_or_create = MagicMock(return_value=('fake_success_result', False))
        with patch('ci.util.extractor.JenkinsBuild') as extractor_jenkins_build, \
                patch('ci.util.extractor.get_downstream_builds_from_build'), \
                patch('ci.util.extractor.get_text_from_url', get_text_from_url), \
                patch('ci.util.extractor.get_json_from_url', get_text_from_url), \
                patch('ci.models.JenkinsResult.objects.get_or_create',
                      jenkins_result_get_or_create):
            extractor_jenkins_build.return_value.number = 42
            extractor_jenkins_build.return_value.timestamp = 'fake_timestamp'
            result = util_jenkins_build.create_jenkins_build_from_url(
                self.dummy_url,
                self.dummy_jenkins_job)
        self.jenkins_build_objects_create.assert_called_with(
            jenkins_job=self.dummy_jenkins_job,
            url=self.dummy_url,
            number=42,
            timestamp='fake_timestamp',
            result='fake_success_result')

    def test_url_is_blacklisted(self):
        url = "http://10.97.2.10:8080/job/share-app-raring-armhf-ci/17/"
        blacklist_jobs = ['raring',]
        with patch('ci.util.jenkins_build.BLACKLIST_JOBS', blacklist_jobs):
            result = util_jenkins_build.create_jenkins_build_from_url(
                url, self.dummy_jenkins_job)
        self.assertIsNone(result)

    def test_extractor_build_barfs_on_us_properly(self):
        url = "http://10.97.2.10:8080/job/share-app-raring-armhf-ci/17/"
        extractor_jenkins_build_mock = MagicMock(side_effect=Exception)
        with patch('ci.util.extractor.JenkinsBuild', extractor_jenkins_build_mock):
            extractor_jenkins_build_mock.return_value.result = None
            result = util_jenkins_build.create_jenkins_build_from_url(
                url, self.dummy_jenkins_job)
        self.assertIsNone(result)

    def test_extractor_build_result_is_none_return_none(self):
        """This probably means the build is running."""

        url = "http://10.97.2.10:8080/job/share-app-raring-armhf-ci/17/"
        with patch('ci.util.extractor.JenkinsBuild') as extractor_jenkins_build_mock:
            extractor_jenkins_build_mock.return_value.result = None
            result = util_jenkins_build.create_jenkins_build_from_url(
                url, self.dummy_jenkins_job)
        self.assertIsNone(result)

    def test_build_duplicate_integrity_error(self):
        util_jenkins_build.create = MagicMock(side_effect=IntegrityError)
        get_coverage_report = MagicMock(side_effect=EnvironmentError)
        jenkins_result_get_or_create = MagicMock(return_value=('fake_success_result', False))
        with patch('ci.util.extractor.JenkinsBuild') as extractor_jenkins_build, \
                patch('ci.util.extractor.get_downstream_builds_from_build_url'), \
                patch('ci.util.extractor.get_coverage_report', get_coverage_report), \
                patch('ci.models.JenkinsResult.objects.get_or_create', jenkins_result_get_or_create):
            extractor_jenkins_build.return_value.number = 42
            extractor_jenkins_build.return_value.timestamp = 'fake_timestamp'
            result = util_jenkins_build.create_jenkins_build_from_url(
                self.dummy_url, self.dummy_jenkins_job)
        self.jenkins_build_objects_create.assert_called_with(
            jenkins_job=self.dummy_jenkins_job,
            url=self.dummy_url,
            number=42,
            timestamp='fake_timestamp',
            result='fake_success_result')
        self.assertIsNone(result)

    def test_extractor_build_result_is_none(self):
        """Weird but extractor returns None for result b/c no builds at all."""

        self.jenkins_build_objects_create = MagicMock(side_effect=IntegrityError)
        jenkins_result_get_or_create = MagicMock(side_effect=IntegrityError)
        with patch('ci.util.extractor.JenkinsBuild') as extractor_jenkins_build, \
                patch('ci.models.JenkinsResult.objects.get_or_create', jenkins_result_get_or_create):
            result = util_jenkins_build.create_jenkins_build_from_url(
                self.dummy_url, self.dummy_jenkins_job)
        self.assertIsNone(result)


class TestCreateDownstreamBuilds(JenkinsBuildManagerTestCase):

    @skip
    def test_vanilla(self):
        get_downstream_builds_from_build = MagicMock(return_value=[extractor.BuildUrl(
            url='http://s-jenkins:8080/job/mir-raring-i386-autolanding/3',
            number=3)])
        extractor_build = MagicMock()
        extractor_build.job_name = 'mir-raring-i386-autolanding'
        extractor_build.number = 3
        jenkins_build_mock = MagicMock(return_value=extractor_build)
        jenkins_job_get = MagicMock(return_value=self.dummy_jenkins_job)
        jenkins_build_get = MagicMock(return_value="fake_downstream_build")
        ci_jenkins_build = MagicMock()
        ci_extractor_build = MagicMock()
        with patch('ci.util.extractor.get_downstream_builds_from_build', get_downstream_builds_from_build), \
                patch('ci.util.extractor.JenkinsBuild', jenkins_build_mock), \
                patch('ci.models.JenkinsJob.objects.get', jenkins_job_get), \
                patch('ci.models.JenkinsBuild.objects.get', jenkins_build_get), \
                patch.object(ci_jenkins_build, 'save') as ci_jenkins_build_save, \
                patch.object(ci_jenkins_build, 'downstream_builds') as ci_jenkins_build_downstream_builds:
            ci_jenkins_build_downstream_builds.add = MagicMock()
            result = self.jenkins_build_manager._create_downstream_builds(
                ci_jenkins_build, ci_extractor_build)
        ci_jenkins_build_downstream_builds.add.assert_called_with(
                "fake_downstream_build")
        ci_jenkins_build_save.assert_called()


    @skip
    def test_jenkins_job_create(self):
        get_downstream_builds_from_build = MagicMock(return_value=[extractor.BuildUrl(
            url='http://s-jenkins:8080/job/mir-raring-i386-autolanding/3',
            number=3)])
        extractor_build = MagicMock()
        extractor_build.job_name = 'mir-raring-i386-autolanding'
        extractor_build.number = 3
        jenkins_build_mock = MagicMock(return_value=extractor_build)
        jenkins_job_get = MagicMock(side_effect=JenkinsJob.DoesNotExist)
        jenkins_build_get = MagicMock(return_value="fake_downstream_build")
        ci_jenkins_build = MagicMock()
        ci_extractor_build = MagicMock()
        ci_extractor_build.url = 'http://s-jenkins:8080/job/fake_jenkins_build/3'
        with patch('ci.util.extractor.get_downstream_builds_from_build', get_downstream_builds_from_build), \
                patch('ci.util.extractor.JenkinsBuild', jenkins_build_mock), \
                patch('ci.models.JenkinsJob.objects.get', jenkins_job_get), \
                patch('ci.models.JenkinsBuild.objects.get', jenkins_build_get), \
                patch.object(ci_jenkins_build, 'save') as ci_jenkins_build_save, \
                patch.object(ci_jenkins_build, 'downstream_builds') as ci_jenkins_build_downstream_builds, \
                patch('ci.util.jenkins.create_jenkins_job_from_name') as jenkins_job_create_from_name:
            ci_jenkins_build_downstream_builds.add = MagicMock()
            result = self.jenkins_build_manager._create_downstream_builds(
                ci_jenkins_build, ci_extractor_build)
        jenkins_job_create_from_name.assert_called_with(
            'http://s-jenkins:8080', name='mir-raring-i386-autolanding')
        ci_jenkins_build_downstream_builds.add.assert_called_with(
                "fake_downstream_build")
        ci_jenkins_build_save.assert_called()


    @skip
    def test_jenkins_build_create(self):
        get_downstream_builds_from_build = MagicMock(return_value=[extractor.BuildUrl(
            url='http://s-jenkins:8080/job/mir-raring-i386-autolanding/3',
            number=3)])
        extractor_build = MagicMock()
        extractor_build.job_name = 'mir-raring-i386-autolanding'
        extractor_build.number = 3
        jenkins_build_mock = MagicMock(return_value=extractor_build)
        jenkins_job_get = MagicMock(return_value=self.dummy_jenkins_job)
        jenkins_build_get = MagicMock(side_effect=JenkinsBuild.DoesNotExist)
        jenkins_build_create_from_name_and_number = MagicMock(return_value='fake_jenkins_build')
        ci_jenkins_build = MagicMock()
        ci_extractor_build = MagicMock()
        ci_extractor_build.url = 'http://s-jenkins:8080/job/fake_jenkins_build/3'
        with patch('ci.util.extractor.get_downstream_builds_from_build', get_downstream_builds_from_build), \
                patch('ci.util.extractor.JenkinsBuild', jenkins_build_mock), \
                patch('ci.models.JenkinsJob.objects.get', jenkins_job_get), \
                patch('ci.models.JenkinsBuild.objects.get', jenkins_build_get), \
                patch.object(ci_jenkins_build, 'save') as ci_jenkins_build_save, \
                patch.object(ci_jenkins_build, 'downstream_builds') as ci_jenkins_build_downstream_builds, \
                patch('ci.models.JenkinsBuild.objects.create_from_name_and_number',
                      jenkins_build_create_from_name_and_number):
            ci_jenkins_build_downstream_builds.add = MagicMock()
            result = self.jenkins_build_manager._create_downstream_builds(
                ci_jenkins_build, ci_extractor_build)
        jenkins_build_create_from_name_and_number.assert_called_with(
            'http://s-jenkins:8080', 'mir-raring-i386-autolanding', 3)
        ci_jenkins_build_downstream_builds.add.assert_called_with(
                "fake_jenkins_build")
        ci_jenkins_build_save.assert_called()


class TestCreateFromNameAndNumber(JenkinsBuildManagerTestCase):

    @skip
    def test_vanilla(self):
        jenkins_build_url = 'http://fake.jenkins.build.url/uhhuh/'
        jenkins_job_get = MagicMock(return_value=self.dummy_jenkins_job)
        extractor_get_build_url = MagicMock(return_value=jenkins_build_url)
        with patch('ci.models.JenkinsJob.objects.get', jenkins_job_get), \
                patch('ci.models.JenkinsBuildManager.create_from_url') as jenkins_build_manager_create_from_url, \
                patch('ci.util.extractor.get_build_url',
                      extractor_get_build_url):
            result = self.jenkins_build_manager.create_from_name_and_number(
                'http://s-jenkins:8080', 'louis-autolanding', 14)
        jenkins_build_manager_create_from_url.assert_called_with(
            jenkins_build_url, self.dummy_jenkins_job)


    @skip
    def test_blacklisted(self):
        blacklist_jobs = ['louis',]
        with patch('ci.models.BLACKLIST_JOBS', blacklist_jobs):
            result = self.jenkins_build_manager.create_from_name_and_number(
                'http://s-jenkins:8080', 'louis-autolanding', 14)
        self.assertIsNone(result)
