# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import copy
from mock import MagicMock, patch
from unittest import skip
from django.test import TestCase
from django.db import IntegrityError
from ci.models import JenkinsJob, JenkinsBuild
import ci.util.extractor as extractor
import ci.util.jenkins_job as util_jenkins_job


class TestCreateJenkinsJobFromUrl(TestCase):

    def setUp(self):
        jenkins_job_objects_create_mock = MagicMock(
            return_value=None)
        self.jenkins_job_objects_create_patch = patch.object(
            JenkinsJob.objects,
            'create',
            new=jenkins_job_objects_create_mock)
        self.jenkins_job_objects_create = self.jenkins_job_objects_create_patch.start()
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()
        self.extractor_job_params = {
            'name': 'fake-autolanding',
            'buildable': True,
            'in_queue': False,
            'last_build': 'fake_last_build',
            'last_completed_build': 'fake_last_completed_build',
            'last_failed_build': 'fake_last_failed_build',
            'last_successful_build': 'fake_last_successful_build'}
        self.expected_job_params = copy.deepcopy(self.extractor_job_params)
        self.dummy_url = "http://this.string.is.actually.a.URL/no/kidding"

    def tearDown(self):
        self.jenkins_job_objects_create_patch.stop()
        self.logger_patch.stop()

    def test_vanilla(self):
        extractor_jenkins_job = MagicMock(
            return_value=MagicMock(**self.extractor_job_params))
        with patch('ci.util.extractor.JenkinsJob', extractor_jenkins_job):
            extractor_jenkins_job.return_value.name = 'fake-autolanding'
            result = util_jenkins_job.create_jenkins_job_from_url(self.dummy_url)
        self.jenkins_job_objects_create.assert_called_with(
            url=self.dummy_url,
            **self.expected_job_params)

    def test_url_is_blacklisted(self):
        url = "http://10.97.2.10:8080/job/share-app-raring-armhf-ci/"
        blacklist_jobs = ['raring',]
        extractor_jenkins_job = MagicMock(
            return_value=MagicMock(**self.extractor_job_params))
        with patch('ci.util.jenkins_job.BLACKLIST_JOBS', blacklist_jobs), \
                patch('ci.util.extractor.JenkinsJob', extractor_jenkins_job):
            result = util_jenkins_job.create_jenkins_job_from_url(url)
        self.assertIsNone(result)

    def test_environment_error_in_extractor(self):
        self.jenkins_job_objects_create.side_effect = EnvironmentError
        with patch('ci.util.extractor.JenkinsJob') as extractor_jenkins_job:
            extractor_jenkins_job.return_value = MagicMock(
                **self.extractor_job_params)
            extractor_jenkins_job.return_value.name = 'fake-autolanding'
            result = util_jenkins_job.create_jenkins_job_from_url(self.dummy_url)
        self.jenkins_job_objects_create.assert_called_with(
            url=self.dummy_url,
            **self.expected_job_params)
        self.assertIsNone(result)

    def test_duplicate_no_update(self):
        self.jenkins_job_objects_create.side_effect = IntegrityError
        jenkins_job_get = MagicMock(return_value="fake_jenkins_job")
        with patch('ci.util.extractor.JenkinsJob') as extractor_jenkins_job, \
                patch('ci.models.JenkinsJob.objects.get', jenkins_job_get):
            extractor_jenkins_job.return_value = MagicMock(
                **self.extractor_job_params)
            extractor_jenkins_job.return_value.name = 'fake-autolanding'
            result = util_jenkins_job.create_jenkins_job_from_url(
                self.dummy_url, update=False)
        self.jenkins_job_objects_create.assert_called_with(
            url=self.dummy_url,
            **self.expected_job_params)
        self.assertEqual('fake_jenkins_job', result)

    def test_duplicate_yes_update(self):
        self.jenkins_job_objects_create.side_effect = IntegrityError
        jenkins_job_get = MagicMock(return_value="fake_jenkins_job")
        with patch('ci.util.extractor.JenkinsJob') as extractor_jenkins_job, \
                patch('ci.models.JenkinsJob.objects.get') as jenkins_job_get, \
                patch('ci.models.JenkinsJob.save') as jenkins_job_save:
            extractor_jenkins_job.return_value = MagicMock(
                **self.extractor_job_params)
            extractor_jenkins_job.return_value.name = 'fake-autolanding'
            result = util_jenkins_job.create_jenkins_job_from_url(
                self.dummy_url, update=True)
        self.jenkins_job_objects_create.assert_called_with(
            url=self.dummy_url,
            **self.expected_job_params)
        jenkins_job_save.assert_called()

    def test_create_builds(self):
        with patch('ci.util.extractor.JenkinsJob') as extractor_jenkins_job, \
                patch('ci.util.jenkins_build.create_jenkins_build_from_url') as \
                        jenkins_build_create_from_url:
            extractor_jenkins_job.return_value = MagicMock(
                **self.extractor_job_params)
            extractor_jenkins_job.return_value.name = 'fake-autolanding'
            dummy_build_url = "http://this.is.a.build.url/duh"
            build = MagicMock(url=dummy_build_url)
            extractor_jenkins_job.return_value.builds = [build, build]
            result = util_jenkins_job.create_jenkins_job_from_url(self.dummy_url,
                                                                   create_builds=True)
        jenkins_build_create_from_url.assert_called_with(dummy_build_url, result)

    def test_create_builds_environment_error(self):
        jenkins_build_create_from_url = MagicMock(side_effect=EnvironmentError)
        with patch('ci.util.extractor.JenkinsJob') as extractor_jenkins_job, \
                patch('ci.util.jenkins_build.create_jenkins_build_from_url',
                      jenkins_build_create_from_url):
            extractor_jenkins_job.return_value = MagicMock(
                **self.extractor_job_params)
            extractor_jenkins_job.return_value.name = 'fake-autolanding'
            dummy_build_url = "http://this.is.a.build.url/duh"
            build = MagicMock(url=dummy_build_url)
            extractor_jenkins_job.return_value.builds = [build, build]
            result = util_jenkins_job.create_jenkins_job_from_url(self.dummy_url,
                                                                   create_builds=True)
        self.jenkins_job_objects_create.assert_called_with(
            url=self.dummy_url,
            **self.expected_job_params)
        jenkins_build_create_from_url.assert_called_with(dummy_build_url, result)


    # TODO this fails on recursion; how to test?
    #def test_create_downstream_jobs(self):
    #    with patch('ci.util.extractor.JenkinsJob') as extractor_jenkins_job, \
    #            patch('ci.models.JenkinsBuild.objects.create_from_url') as \
    #                    jenkins_build_create_from_url:
    #        extractor_jenkins_job.return_value = MagicMock(
    #            **self.extractor_job_params)
    #        extractor_jenkins_job.return_value.name = 'fake-autolanding'
    #        dummy_downstream_job_url = "http://this.is.a.build.url/duh"
    #        downstream_job = MagicMock(url=dummy_downstream_job_url)
    #        extractor_jenkins_job.return_value.downstream_jobs = [downstream_job,
    #                                                              downstream_job]
    #        result = jenkins_utils.create_from_url(self.dummy_url)
    #    jenkins_utils.create.assert_called_with(
    #        url=self.dummy_url,
    #        **self.extractor_job_params)
    #    jenkins_build_create_from_url.assert_called_with(dummy_build_url, result)


    # TODO again how to test with recursion
    #def test_create_downstream_jobs_extractor_exception(self):
    #    with patch('ci.util.extractor.JenkinsJob') as extractor_jenkins_job, \
    #            patch('ci.models.JenkinsBuild.objects.create_from_url') as \
    #                    jenkins_build_create_from_url:
    #        extractor_jenkins_job.return_value = MagicMock(
    #            **self.extractor_job_params)
    #        extractor_jenkins_job.return_value.name = 'fake-autolanding'
    #        downstream_job = MagicMock(side_effect=EnvironmentError)
    #        extractor_jenkins_job.return_value.downstream_jobs = [downstream_job,
    #                                                              downstream_job]
    #        result = jenkins_utils.create_from_url(self.dummy_url)
    #    jenkins_utils.create.assert_called_with(
    #        url=self.dummy_url,
    #        **self.extractor_job_params)
    #    jenkins_build_create_from_url.assert_called_with(dummy_build_url, result)



class TestCreateFromName(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()
        jenkins_job_mock = MagicMock(
            spec=JenkinsJob,
            return_value=None)
        self.jenkins_job_patch = patch.object(
            JenkinsJob,
            '__init__',
            new=jenkins_job_mock)
        self.jenkins_job_patch.start()
        self.jenkins_job = \
            JenkinsJob()

    def tearDown(self):
        self.logger_patch.stop()
        self.jenkins_job_patch.stop()

    def test_vanilla(self):
        dummy_job_name = 'dbus-test-runner-autolanding'
        dummy_jenkins_job = 'perfectly_legitimate_jenkins_job'
        extractor_get_job_url = MagicMock(return_value=dummy_jenkins_job)
        with patch('ci.util.extractor.get_job_url', extractor_get_job_url), \
                patch('ci.util.jenkins_job.create_jenkins_job_from_url') as \
                    create_jenkins_job_from_url:
            result = util_jenkins_job.create_jenkins_job_from_name(
                'http://s-jenkins:8080', dummy_job_name)
        extractor_get_job_url.assert_called_with('http://s-jenkins:8080',
                                                 dummy_job_name)
        create_jenkins_job_from_url.assert_called_with(
            dummy_jenkins_job)
