# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mock import MagicMock, patch
from django.test import TestCase
from django.db import IntegrityError
from ci.models import JenkinsJob, JenkinsRun
import ci.util.extractor as extractor
from ci.util.jenkins_run import create_jenkins_run_from_url


class TestCreateFromUrl(TestCase):

    def setUp(self):
        self.logger_patch = patch('ci.models.logger')
        self.logger_patch.start()
        self.fake_url = "http://this.string.is.actually.a.URL/no/kidding"
        self.fake_jenkins_build = "a Django representation of a Jenkins job"

    def tearDown(self):
        self.logger_patch.stop()

    def test_vanilla(self):
        extractor_jenkins_run = MagicMock()
        jenkins_flavor_get_or_create = MagicMock(return_value=(
                'fake_jenkins_flavor', False))
        series_get_or_create = MagicMock(return_value=(
                'fake_series', False))
        jenkins_result_get_or_create = MagicMock(return_value=(
                'fake_jenkins_result', False))
        with patch('ci.util.extractor.JenkinsRun', extractor_jenkins_run), \
                patch('ci.models.JenkinsFlavor.objects.get_or_create',
                      jenkins_flavor_get_or_create), \
                patch('ci.models.Series.objects.get_or_create',
                      series_get_or_create), \
                patch('ci.models.JenkinsResult.objects.get_or_create',
                      jenkins_result_get_or_create), \
                patch('ci.models.JenkinsRun.objects.create') as jenkins_run_objects_create:
            extractor_jenkins_run.return_value.number = 42
            extractor_jenkins_run.return_value.name = 'fake_jenkins_run_name'
            duration_mock = MagicMock()
            duration_mock.seconds = 3.1415
            extractor_jenkins_run.return_value.duration = duration_mock
            extractor_jenkins_run.return_value.timestamp = 'fake_timestamp'
            extractor_jenkins_run.return_value.parameters = 'fake_parameters'
            result = create_jenkins_run_from_url(
                self.fake_url, self.fake_jenkins_build)
        jenkins_run_objects_create.assert_called_with(
            jenkins_build=self.fake_jenkins_build,
            number=42,
            name='fake_jenkins_run_name',
            url=self.fake_url,
            duration=3.1415,
            timestamp='fake_timestamp',
            flavor='fake_jenkins_flavor',
            series='fake_series',
            result='fake_jenkins_result')

    def test_extractor_build_result_is_none(self):
        """Weird but extractor returns None for result b/c no runs at all."""

        self.fake_url = "http://this.string.is.actually.a.URL/no/kidding"
        self.fake_jenkins_build = "a Django representation of a Jenkins build"
        extractor_jenkins_run = MagicMock()
        jenkins_flavor_get_or_create = MagicMock(return_value=(
                'fake_jenkins_flavor', False))
        series_get_or_create = MagicMock(return_value=(
                'fake_series', False))
        jenkins_result_get_or_create = MagicMock(side_effect=IntegrityError)
        with patch('ci.util.extractor.JenkinsRun', extractor_jenkins_run), \
                patch('ci.models.JenkinsFlavor.objects.get_or_create',
                      jenkins_flavor_get_or_create), \
                patch('ci.models.Series.objects.get_or_create', series_get_or_create), \
                patch('ci.models.JenkinsResult.objects.get_or_create', jenkins_result_get_or_create), \
                patch('ci.models.JenkinsRun.objects.create') as jenkins_run_objects_create:
            result = create_jenkins_run_from_url(
                self.fake_url, self.fake_jenkins_build)
        self.assertIsNone(result)

    def test_run_duplicate_integrity_error(self):
        self.fake_url = "http://this.string.is.actually.a.URL/no/kidding"
        self.fake_jenkins_build = "a Django representation of a Jenkins job"
        extractor_jenkins_run = MagicMock()
        jenkins_flavor_get_or_create = MagicMock(return_value=(
                'fake_jenkins_flavor', False))
        series_get_or_create = MagicMock(return_value=(
                'fake_series', False))
        jenkins_result_get_or_create = MagicMock(return_value=(
                'fake_jenkins_result', False))
        with patch('ci.util.extractor.JenkinsRun', extractor_jenkins_run), \
                patch('ci.models.JenkinsFlavor.objects.get_or_create',
                      jenkins_flavor_get_or_create), \
                patch('ci.models.Series.objects.get_or_create',
                      series_get_or_create), \
                patch('ci.models.JenkinsResult.objects.get_or_create',
                      jenkins_result_get_or_create), \
                patch('ci.models.JenkinsRun.objects.create') as jenkins_run_objects_create:
            jenkins_run_objects_create.side_effect = IntegrityError
            extractor_jenkins_run.return_value.number = 42
            extractor_jenkins_run.return_value.name = 'fake_jenkins_run_name'
            duration_mock = MagicMock()
            duration_mock.seconds = 3.1415
            extractor_jenkins_run.return_value.duration = duration_mock
            extractor_jenkins_run.return_value.timestamp = 'fake_timestamp'
            extractor_jenkins_run.return_value.parameters = 'fake_parameters'
            result = create_jenkins_run_from_url(
                self.fake_url, self.fake_jenkins_build)
        jenkins_run_objects_create.assert_called_with(
            jenkins_build=self.fake_jenkins_build,
            number=42,
            name='fake_jenkins_run_name',
            url=self.fake_url,
            duration=3.1415,
            timestamp='fake_timestamp',
            flavor='fake_jenkins_flavor',
            series='fake_series',
            result='fake_jenkins_result')
        self.assertIsNone(result)
