# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Defines heuristics to report more user friendly views of the raw data"""


def get_integration_status(JenkinsJob, JenkinsBuild, build):
    """Determines the published status given a build instance of an
    integration job"""
    build_set = build.downstream_builds.all()
    status = 'Published'

    # Set status according to the last failed build in the workflow,
    # assumes that the builds are organized in the workflow order.
    for b in build_set:
        if 'prepare' in b.jenkins_job.name:
            if b.result.name == 'UNSTABLE':
                status = 'Published (with some components ignored)'
                break
            if b.result.name != 'SUCCESS':
                status = 'Prepare failed'
                break
        if 'build' in b.jenkins_job.name:
            if b.result.name != 'SUCCESS':
                status = 'Build failed'
                break
        if 'check' in b.jenkins_job.name:
            if b.result.name != 'SUCCESS':
                status = 'Testing failed'
                break
        if 'publish' in b.jenkins_job.name:
            if b.result.name != 'SUCCESS':
                status = 'Failed publish'
                break

    # If the stack was not automatically published, check the timestamp of
    # the publish job for that stack. If a newer successful build exists,
    # then assume it was manually published.
    # TODO: Do we need to check the user who started the build?
    if "Published" not in status:
        jobs = JenkinsJob.objects.filter(upstream_job=build.jenkins_job)
        for j in jobs:
            if 'publish' in j.name:
                publish_build = JenkinsBuild.objects.get(
                    jenkins_job=j, number=j.last_successful_build)
                if build.timestamp < publish_build.timestamp:
                    status = 'Manual publish'
    return status


def get_integration_last_publish(JenkinsJob, JenkinsBuild, integration_job):
    """Returns the timestamp of the last publish job for a stack derived
    from the integration job"""
    jobs = JenkinsJob.objects.filter(upstream_job=integration_job)

    # Search for the 'publish' job referenced by the integration job,
    # then find the last successful build for that job.
    for j in jobs:
        if 'publish' in j.name:
            build = JenkinsBuild.objects.get(jenkins_job=j,
                                             number=j.last_successful_build)
            if build:
                return build.timestamp
    return None


def get_integration_last_test_counts(JenkinsJob, JenkinsBuild, integration_job):
    """Returns the test_report of the last check and autopilt job for a
    stack derived from the integration job"""
    jobs = JenkinsJob.objects.filter(upstream_job=integration_job)

    # Search for the 'publish' job referenced by the integration job,
    # then find the last successful build for that job.
    for j in jobs:
        if 'check' in j.name:
            known_build_ids = set(
                [build.number for build in JenkinsBuild.objects.filter(
                    jenkins_job=j)])
            known_build_ids = list(known_build_ids)
            known_build_ids = sorted(known_build_ids)
            for build_number in known_build_ids:
                build = JenkinsBuild.objects.get(jenkins_job=j,
                                                 number=build_number)
                for db in build.downstream_builds.all():
                    if 'autopilot' in db.jenkins_job.name:
                        return build.test_report
    return None


def get_integration_last_pass_count(JenkinsJob, JenkinsBuild, integration_job):
    report = get_integration_last_test_counts(JenkinsJob, JenkinsBuild,
                                              integration_job)
    if report:
        return report.pass_count
    return None


def get_integration_last_fail_count(JenkinsJob, JenkinsBuild, integration_job):
    report = get_integration_last_test_counts(JenkinsJob, JenkinsBuild,
                                              integration_job)
    if report:
        return report.fail_count
    return None


def get_integration_last_total_count(JenkinsJob, JenkinsBuild, integration_job):
    report = get_integration_last_test_counts(JenkinsJob, JenkinsBuild,
                                              integration_job)
    if report:
        return report.pass_count + report.fail_count + report.skip_count
    return None
