# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
from django.db import transaction, IntegrityError
from ci.models import (JenkinsJob,
                       JenkinsBuild,
                       JenkinsResult,
                       JenkinsCoverageReport,
                       JenkinsTestReport)
import ci.util.extractor as extractor
from ci.util.coverage_report import create_coverage_report_from_url
from ci.util.jenkins_test_report import create_jenkins_test_report_from_url
from ci.util.jenkins_run import create_jenkins_run_from_url

logger = logging.getLogger('qa_dashboard')

BLACKLIST_JOBS = [
    'generic-dput',
    'generic-land',
    'generic-update_mp',
    'phablet-dput',
    'phablet-land',
    'generic-mediumtests',
    'generic-cleanup',
    'generic-release-land',
    '1.1prepare',
]

def create_downstream_builds(ci_jenkins_build,
                             ci_extractor_build):
    server = extractor.get_server_url(ci_extractor_build.url)
    downstream_builds = extractor.get_downstream_builds_from_build(
        ci_extractor_build)
    for downstream_build in downstream_builds:
        try:
            extractor_build = extractor.JenkinsBuild(downstream_build.url)
        except EnvironmentError, e:
            logger.info("EnvironmentError: {}".format(e))
            continue
        job_name = extractor_build.job_name
        build_number = extractor_build.number
        logger.debug("downstream job: {}\tbuild: {}".format(job_name,
                                                            build_number))
        # make sure we have the JenkinsJob
        try:
            jenkins_job = JenkinsJob.objects.get(name=job_name)
        except JenkinsJob.DoesNotExist:
            logger.debug("Need to create job: {}".format(job_name))
            # TODO reexamine circular import problem
            from ci.util.jenkins_job import create_jenkins_job_from_name
            jenkins_job = create_jenkins_job_from_name(server,
                                                       name=job_name)
            if jenkins_job is not None:
                jenkins_job.upstream_job = ci_jenkins_build.jenkins_job
                jenkins_job.save()
        # now get or create the downstream JenkinsBuild
        # TODO this could be recursive
        try:
            downstream_jenkins_build = JenkinsBuild.objects.get(
                jenkins_job=jenkins_job,
                number=build_number)
        except JenkinsBuild.DoesNotExist:
            downstream_jenkins_build = \
                    create_jenkins_build_from_name_and_number(
                server, job_name, build_number)
        if downstream_jenkins_build:
            ci_jenkins_build.downstream_builds.add(downstream_jenkins_build)
    ci_jenkins_build.save()


# FIXME: restore support for multi-configuration builds
def create_downstream_runs(ci_jenkins_build, extractor_build):
    logger.debug("importing runs associated with JenkinsBuild at {}".format(extractor_build.url))
    for run in extractor_build.runs:
        pass
        # FIXME: retore this, it breaks within
        #jenkins_run = create_jenkins_run_from_url(run.url,
        #                                          ci_jenkins_build)

def create_cobertura_report(jenkins_build, extractor_build):
    """Get a Cobertura coverage report associated with a build url.
    
    Just return None if there's no report.
    """

    # get a Cobertura coverage report, if it exists
    logger.debug("Slurping Cobertura reports.")
    try:
        coverage_url = extractor_build.get_coverage_report_url()
        jenkins_coverage_report = create_coverage_report_from_url(
            jenkins_build, coverage_url)
        jenkins_build.coverage_report = jenkins_coverage_report
        jenkins_build.save()
        logger.info("Cobertura created: {}.".format(jenkins_build.number))
    except EnvironmentError, e:
        # no biggie, we just don't have a Cobertura report
        logger.info("No Cobertura data: {}".format(e))
    except IntegrityError, e:
        logger.info("Cobertura duplicate encountered.")
        #transaction.rollback()
        pass

def aggregate_coverage_report(jenkins_build):
    # TODO use django aggregation?
    logger.debug("Aggegate coverage reports: %s" % jenkins_build)
    if jenkins_build.coverage_report:
        # Don't aggregate when the build already has a coverage report
        return
    report_count = 0
    line_count = 0
    hit_count = 0
    branch_count = 0
    taken_count = 0
    total_count = 0
    for b in jenkins_build.downstream_builds.all():
        if b.coverage_report:
            report_count += 1
            line_count += b.coverage_report.line_count
            hit_count += b.coverage_report.hit_count
            branch_count += b.coverage_report.branch_count
            taken_count += b.coverage_report.taken_count
            total_count += b.coverage_report.total_count
    if report_count > 0:
        # Only add an aggregate object if others were found
        jenkins_build.coverage_report = JenkinsCoverageReport.objects.create(
            jenkins_build=jenkins_build,
            line_count=line_count,
            hit_count=hit_count,
            branch_count=branch_count,
            taken_count=taken_count,
            total_count=total_count)
        jenkins_build.save()


def create_jenkins_test_report(jenkins_build, extractor_build):

    logger.debug("Slurping Test Reports.")
    try:
        test_url = extractor_build.get_test_report_url()
        jenkins_test_report = create_jenkins_test_report_from_url(
            jenkins_build, test_url)
        jenkins_build.test_report = jenkins_test_report
        jenkins_build.save()
        logger.info("Test Report created: {}.".format(jenkins_build.number))
    except EnvironmentError, e:
        # no data if the build failed, e.g.
        logger.info("No test report data found: {}".format(e))
    except IntegrityError, e:
        logger.info("Test Report duplicate encountered.")
        logger.debug(e)
        transaction.rollback()


def aggregate_test_report(jenkins_build):
    # TODO use django aggregation?
    logger.debug("Aggegate test reports: %s" % jenkins_build)
    if jenkins_build.test_report:
        # Don't aggregate when the build already has a test report
        return
    fail_count = 0
    pass_count = 0
    skip_count = 0
    duration = 0
    count = 0
    for b in jenkins_build.downstream_builds.all():
        if b.test_report:
            count += 1
            fail_count += b.test_report.fail_count
            pass_count += b.test_report.pass_count
            skip_count += b.test_report.skip_count
            duration += b.test_report.duration
    if count > 0:
        # Only add an aggregate object if others were found 
        jenkins_build.test_report = JenkinsTestReport.objects.create(
            pass_count=pass_count,
            fail_count=fail_count,
            skip_count=skip_count,
            duration=duration)
        jenkins_build.save()


def create_jenkins_build_from_url(url, jenkins_job=None):
    """Create a Jenkins build in our Django database given its URL.

    :param url: the URL of the Jenkins build
    :type url: string
    :param jenkins_job: JenkinsJob of which the URL is a build
    :type jenkins_job: Django JenkinsJob
    :rtype: Django JenkinsBuild
    """

    logger.debug("JenkinsBuildManager creating build from url {}.".format(url))
    for name in BLACKLIST_JOBS:
        if name in url:
            logger.info("Skipping blacklisted job at {}.".format(url))
            return None
    try:
        extractor_build = extractor.JenkinsBuild(url)
    except Exception, e:
        transaction.rollback()
        logger.error("Exception: {}".format(e))
        return None
    try:
        if extractor_build.result is None:
            # This build is actively running, no need to slurp it
            logger.info("Skipping build at {}: may be actively running.".format(url))
            transaction.rollback()
            return None
    except Exception, e:
        # in some cases this 404s; same as above, no need to slurp a broken build
        logger.error("Exception while sluring build at {}: {}".format(url, e))
        transaction.rollback()
        return None
    
    try:
        result, created = JenkinsResult.objects.get_or_create(name=extractor_build.result)
    except IntegrityError, e:
        # no builds yet, IntegrityError when attempting to create JenkinsResult with None name
        transaction.rollback()
        logger.debug("No builds yet.")
        return None
    
    try:
        logger.debug("Creating JenkinsBuild at {}.".format(url))
        if jenkins_job is None:
            logger.warning("Creating JenkinsBuild at {} with no parent job.".format(url))
        jenkins_build = JenkinsBuild.objects.create(
            jenkins_job=jenkins_job,
            url=url,
            number=extractor_build.number,
            timestamp=extractor_build.timestamp,
            result=result)
    except IntegrityError:
        transaction.rollback()
        logger.info("Duplicate Jenkins build found at {}".format(url))
        return None

    create_downstream_runs(jenkins_build, extractor_build)
    create_downstream_builds(jenkins_build, extractor_build)
    create_cobertura_report(jenkins_build, extractor_build)
    create_jenkins_test_report(jenkins_build, extractor_build)
    aggregate_coverage_report(jenkins_build)
    aggregate_test_report(jenkins_build)
    return jenkins_build


def create_jenkins_build_from_name_and_number(server, name, number):
    """Create a Jenkins build in our Django database given a server, name, and number.

    :param server: URL of the Jenkins server
    :type server: string
    :param name: Jenkins build name
    :type name: string 
    :param number: the number of the build
    :type number: int
    :rtype: Django JenkinsBuild
    """

    for blacklist in BLACKLIST_JOBS:
        if blacklist in name:
            logger.info("Skipping blacklisted build {} #{}.".format(name, number))
            return None
    logger.debug("Creating JenkinsBuild for {} #{}.".format(name, number))
    try:
        jenkins_job = JenkinsJob.objects.get(name=name)
    except JenkinsJob.DoesNotExist:
        return None
    url = extractor.get_build_url(server, name, number)
    return create_jenkins_build_from_url(url, jenkins_job)
