# QA Dashboard
# Copyright 2012-2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging

from django.conf import settings
from django.db import transaction, IntegrityError

from ci.models import JenkinsJob, JenkinsBuild
import ci.util.extractor as extractor
import ci.util.jenkins_build as util_jenkins_build
from jenkinsapi.api import Jenkins
from jenkinsapi.exceptions import UnknownJob


class CiException(Exception):
    pass

logger = logging.getLogger('qa_dashboard')

BLACKLIST_JOBS = [
    'generic-dput',
    'generic-land',
    'generic-update_mp',
    'phablet-dput',
    'phablet-land',
    'generic-mediumtests',
    'generic-cleanup',
    'generic-release-land',
    '1.1prepare',
]


@transaction.autocommit
def create_jenkins_job_from_url(url,
                                update=False,
                                create_builds=True,
                                create_downstream_jobs=True):
    """Create a Jenkins job in our Django database given its URL.

    :param url: the URL of the Jenkins job
    :type url: string
    :param update: update the job if it already exists in our db
    :type update: boolean
    :param create_builds: also slurp the Jenkins job's builds
    :type create_builds: boolean
    :param create_downstream_jobs: also slurp the Jenkins job's downstream jobs
    :type boolean:
    :rtype: Django JenkinsJob
    """

    logger.info("JenkinsJob create_from_url: %s" % (url))
    for name in BLACKLIST_JOBS:
        if name in url:
            logger.info("Skipping blacklisted job {}.".format(name))
            return None
    try:
        extractor_job = extractor.JenkinsJob(url)
        params = {'name': extractor_job.name,
                  'url': url,
                  'buildable': extractor_job.buildable,
                  'in_queue': extractor_job.in_queue,
                  'last_build': extractor_job.last_build,
                  'last_completed_build': extractor_job.last_completed_build,
                  'last_failed_build': extractor_job.last_failed_build,
                  'last_successful_build': extractor_job.last_successful_build}
        jenkins_job = JenkinsJob.objects.create(**params)
    except EnvironmentError, e:
        # No job found: this can happen when a project is defined in a stack but
        # the jobs have not yet been deployed
        transaction.rollback()
        logger.debug(e)
        return None
    except IntegrityError, e:
        # uniqueness constraint violated
        transaction.rollback()
        logger.debug(e)
        jenkins_job = JenkinsJob.objects.get(name=extractor_job.name)
        if update:
            logger.debug("Updating job from {}".format(url))
            jenkins_job.update(**params)
    if create_downstream_jobs:
        try:
            for downstream_extractor_job in extractor_job.downstream_jobs:
                # TODO: protect against circular refs, just sayin'
                downstream_jenkins_job = create_jenkins_job_from_url(
                    downstream_extractor_job.url)
                logger.info("downstream job created: {}".format(downstream_jenkins_job))
                if downstream_jenkins_job:
                    downstream_jenkins_job.upstream_job = jenkins_job
                    downstream_jenkins_job.project = jenkins_job.project
                    downstream_jenkins_job.save()
        except EnvironmentError, e:
            logger.info("EnvironmentError: {}".format(e))
    if create_builds:
        for build in extractor_job.builds:
            try:
                jenkins_build = util_jenkins_build.create_jenkins_build_from_url(
                    build.url, jenkins_job)
                # DEBUGGING: Hack, only return 1 build to speed things up
                #break
            except EnvironmentError, e:
                logger.info("EnvironmentError: {}".format(e))
    return jenkins_job


def create_jenkins_job_from_name(server, name):
    logger.debug("Creating JenkinsJob from name {}.".format(name))
    return create_jenkins_job_from_url(extractor.get_job_url(server, name))


def latest_builds(jenkins_job):
    """Return an iterator of builds that are on Jenkins but not in our DB."""

    ps_jenkins = Jenkins(settings.JENKINS_URL, )
    try:
        jenkinsapi_job = ps_jenkins.get_job(jenkins_job.name)
    except UnknownJob:
        raise CiException("Job {} not known on {}.".format(
                jenkins_job.name, settings.JENKINS_URL))
    jenkinsapi_job_build_ids = set(jenkinsapi_job.get_build_ids())
    known_build_ids = set([build.number for build in JenkinsBuild.objects.filter(
        jenkins_job=jenkins_job)])
    unknown_build_ids = (jenkinsapi_job_build_ids - known_build_ids)
    for build_id in list(unknown_build_ids):
        yield jenkinsapi_job.get_build(build_id)


def update_jenkins_job(jenkins_job):
    """Updates a JenkinsJob with latest data from Jenkins."""

    for jenkinsapi_build in latest_builds(jenkins_job):
        build = util_jenkins_build.create_jenkins_build_from_url(
            jenkinsapi_build.baseurl, jenkins_job)
        if not build:
            logger.error("Failed to create build for {}.".format(jenkins_job))
